"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const inquirer = require("inquirer");
const execa = require("execa");
const Listr = require("listr");
const UpdateRenderer = require('listr-update-renderer');
const fs = require('fs-extra');
exports.configProject = {
    projectType: ''
};
exports.configWebComponent = {
    componentType: 'string',
    componentName: '',
    componentDescription: '',
    compName: '',
    shortenedNameComp: '',
    shortenedName: ''
};
class NewProject extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(NewProject);
        this.printBanner();
        await this.askProjectType();
    }
    printBanner() {
        this.log(chalk_1.default.bold.yellow('         #######  ########  ######## ##    ##    '));
        this.log(chalk_1.default.bold.yellow('        ##     ## ##     ## ##       ###   ##    '));
        this.log(chalk_1.default.bold.yellow('        ##     ## ##     ## ##       ####  ##    '));
        this.log(chalk_1.default.bold.yellow('        ##     ## ########  ######   ## ## ##    '));
        this.log(chalk_1.default.bold.yellow('        ##     ## ##        ##       ##  ####    '));
        this.log(chalk_1.default.bold.yellow('        ##     ## ##        ##       ##   ###    '));
        this.log(chalk_1.default.bold.yellow('         #######  ##        ######## ##    ##     '));
        this.log(chalk_1.default.bold.yellow(''));
        this.log(chalk_1.default.bold.yellow(' ######   #######  ##     ## ########   #######  ##    ## ######## ##    ## ########  ######'));
        this.log(chalk_1.default.bold.yellow('##    ## ##     ## ###   ### ##     ## ##     ## ###   ## ##       ###   ##    ##    ##    ##'));
        this.log(chalk_1.default.bold.yellow('##       ##     ## #### #### ##     ## ##     ## ####  ## ##       ####  ##    ##    ##'));
        this.log(chalk_1.default.bold.yellow('##       ##     ## ## ### ## ########  ##     ## ## ## ## ######   ## ## ##    ##     ######'));
        this.log(chalk_1.default.bold.yellow('##       ##     ## ##     ## ##        ##     ## ##  #### ##       ##  ####    ##          ##'));
        this.log(chalk_1.default.bold.yellow('##    ## ##     ## ##     ## ##        ##     ## ##   ### ##       ##   ###    ##    ##    ##'));
        this.log(chalk_1.default.bold.yellow(' ######   #######  ##     ## ##         #######  ##    ## ######## ##    ##    ##     ######'));
        this.log();
        this.log();
    }
    askProjectType() {
        const questions = [
            {
                type: 'list',
                name: 'projectType',
                message: 'Do you want to create a Web Component or an SPA?',
                choices: ['SPA', 'Web Component'],
                filter: function (val) {
                    let clean = val.replace(/ /g, '');
                    return clean.toUpperCase();
                }
            }
        ];
        inquirer.prompt(questions)
            .then(answer => {
            exports.configProject = answer;
            switch (exports.configProject.projectType) {
                case 'SPA':
                    this.log('It is going to create an archetype of Angular 7 Single Page Application');
                    this.askAngularAppConfig();
                    break;
                case 'WEBCOMPONENT':
                    this.log('It is going to create an archetype of Web Component based on Angular 7');
                    this.askComponentType();
            }
        })
            .catch(error => {
            this.log('An error has occurred.');
        });
    }
    async askComponentType() {
        const questions = [
            {
                type: 'list',
                name: 'componentType',
                message: 'You must choose one option: UI (presentation component), API (data access component) or BS (business logic component)',
                choices: ['UI', 'API', 'BS'],
                filter: function (val) {
                    return val.toUpperCase();
                }
            },
            {
                type: 'input',
                name: 'componentName',
                message: 'Choose a name for your component, just a word that defines its functionality:',
            },
            {
                type: 'input',
                name: 'componentDescription',
                message: 'Describe the functionality that the component covers with one or more sentences:',
            },
        ];
        await inquirer.prompt(questions)
            .then(answer => {
            exports.configWebComponent = answer;
            exports.configWebComponent.compName = 'webcomponent-' + exports.configWebComponent.componentType.toLowerCase() + '-' + exports.configWebComponent.componentName;
            this.log(chalk_1.default.bold.redBright('This is the information of your component:'));
            this.log(chalk_1.default.bold.cyan('Name: '));
            this.log(chalk_1.default.green(exports.configWebComponent.compName));
            this.log(chalk_1.default.bold.cyan('Description: '));
            this.log(chalk_1.default.green(exports.configWebComponent.componentDescription));
            //TODO: Añadir una llamada a GitLab aquí para que vaya a comprobar si el repositorio existe.
            this.log('');
            this.log(chalk_1.default.bold.red('[ATTENTION] The generation of the archetype begins!'));
            this.generateComponentArchetype(exports.configWebComponent);
            // this.probandoTutorial();
        });
    }
    askAngularAppConfig() {
        this.log(chalk_1.default.bold.red('This functionality is in development. Sorry for the inconvenience.'));
        this.log('');
        this.askProjectType();
    }
    generateComponentArchetype(configWebComponent) {
        new Listr([
            {
                title: 'Clonando repositorio del arquetipo...',
                task: () => {
                    execa.stdout('git', ['clone', 'https://github.com/hedesil/webcomponent-angular-archetype', '--progress', '--verbose'])
                        .then(res => {
                        console.log(res);
                        this.renameFiles(configWebComponent);
                        this.renameInsideFiles(configWebComponent);
                        this.renameFolders(configWebComponent);
                    })
                        .catch(err => {
                        this.log('Ha ocurrido un error :( ' + err);
                    });
                }
            },
        ], {
            concurrent: false,
            renderer: UpdateRenderer,
        }).run();
    }
    async renameFiles(configWebComponent) {
        fs.rename('./webcomponent-angular-archetype/src/app/shortenedname/shortenedname.component.css', './webcomponent-angular-archetype/src/app/shortenedName/' + configWebComponent.componentName + '.component.css', function (err) {
            if (err)
                console.log('Error: ' + err);
            console.log(chalk_1.default.bold.greenBright('File Renamed to: ' + configWebComponent.componentName + '.component.css'));
        });
        fs.rename('./webcomponent-angular-archetype/src/app/shortenedname/shortenedname.component.spec.ts', './webcomponent-angular-archetype/src/app/shortenedName/' + configWebComponent.componentName + '.component.spec.ts', function (err) {
            if (err)
                console.log('Error: ' + err);
            console.log(chalk_1.default.bold.greenBright('File Renamed to: ' + configWebComponent.componentName + '.component.spec.ts'));
        });
        fs.rename('./webcomponent-angular-archetype/src/app/shortenedname/shortenedname.component.ts', './webcomponent-angular-archetype/src/app/shortenedName/' + configWebComponent.componentName + '.component.ts', function (err) {
            if (err)
                console.log('Error: ' + err);
            console.log(chalk_1.default.bold.greenBright('File Renamed to: ' + configWebComponent.componentName + '.component.ts'));
        });
        fs.rename('./webcomponent-angular-archetype/src/app/shortenedname/shortenedname.component.html', './webcomponent-angular-archetype/src/app/shortenedName/' + configWebComponent.componentName + '.component.html', function (err) {
            if (err)
                console.log('Error: ' + err);
            console.log(chalk_1.default.bold.greenBright('File Renamed to: ' + configWebComponent.componentName + '.component.html'));
        });
    }
    async renameFolders(configWebComponent) {
        fs.renameSync('./webcomponent-angular-archetype/src/app/shortenedname', './webcomponent-angular-archetype/src/app/' + configWebComponent.componentName);
        fs.rename('./webcomponent-angular-archetype', ('./' + configWebComponent.compName), function (err) {
            if (err) {
                console.log('Error: ' + err);
            }
            console.log('Directory Renamed to: ' + configWebComponent.compName);
        });
    }
    async renameInsideFiles(configWebComponent) {
        const replace = require("replace");
        replace({
            regex: '<compName>',
            replacement: configWebComponent.compName,
            paths: ['./webcomponent-angular-archetype'],
            recursive: true,
            silent: true
        });
        replace({
            regex: '<shortenedName>',
            replacement: configWebComponent.componentName,
            paths: ['./webcomponent-angular-archetype'],
            recursive: true,
            silent: true
        });
        replace({
            regex: '<shortenedNameComp>',
            replacement: configWebComponent.componentName.charAt(0).toUpperCase() + configWebComponent.componentName.slice(1) + 'Component',
            paths: ['./webcomponent-angular-archetype'],
            recursive: true,
            silent: true
        });
        replace({
            regex: '<componentDescription>',
            replacement: configWebComponent.componentDescription,
            paths: ['./webcomponent-angular-archetype'],
            recursive: true,
            silent: true
        });
    }
}
NewProject.description = 'Add new project based on Angular 7 and Web Components standard';
NewProject.flags = {
    help: command_1.flags.help({ char: 'h' })
};
exports.default = NewProject;
